// THIS IS THE HEADER FILE USED BY main.c

/*--------------------------------------
  INCLUDE SOME STANDARD C HEADER FILES
  --------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

/*--------------------------------------------------------------------
  DEFINED USED PHYSICAL CONSTANTS
  --------------------------------------------------------------------*/

#define CI 2.998e+10          // SPEED OF LIGHT 
#define Planck 6.626e-27    // PLANCK'S CONSTANT
#define Me 9.1096e-28       // ELECTRON MASS 
#define Qe 4.8032e-10       // ELECTRON CHARGE 
#define PI 3.14159          // PI
#define Bcr 2.943e13        // CRITICAL MAGNETIC FIELD IN GAUSS MULTIPLIED BY 2/3 TO AGREE WITH CRUSIUS & SCHLICKEISER
#define SigmaT 6.65e-25     // THOMPSON CROSS SECTION
#define Re 2.818e-13        // CLASSICAL ELECTRON RADIUS 
#define Kb 1.381e-16         // BOLTZMANN CONSTANT
#define alpha 7.565e-15     // BLACK BODY CONSTANT U=ALPHA T^4 
#define Tcmb 2.73           // CMB TEMPERATURE AT z=0

/*------------------------------------------------
 FUNCTIONS                       
 ------------------------------------------------*/

/* READS THE PHYSICAL AND COMPUTATIONAL PARAMETERS REQUIRED FROM THE FILE 
   "datafiles/Initial_Values" */

void Initial_Values();

/* ALLOCATES SPACE FOR ALL THE GLOBAL ARRAYS USED IN THE PROGRAM */

void AllocateSpace();

/* FREES THE  SPACE FOR  ALL THE GLOBAL ARRAYS USED IN THE PROGRAM */

void FreeSpace();
 
/* INITIALIZES CONSTANTS AND ARRAYS TO BE USED THROUGHOUT THE PROGRAM */

void Initialize();

/* CALCULATES THE CONTINUOUS ELECTRON ENERGY LOSSES */

double Gamma_Dot(int j);


 /* CALCULATE THE ELECTRON DISTRIBUTION, THE EMISSIVITIES AND ADVANCE THE 
    SOLUTION IN TIME WITH AN ADAPTIVE TIMESTEP, UP TO THE POINT WHERE THE 
    SOLUTION HAS CONVERGED TO A STEADY-STATE. AT EACH TIME STEP WRITE
    THE RESULTS IN A SERIES OF FILES TO BE FOUND IN THE DIRECTORY datafiles.
    ROUTINES FOR PLOTTING THE DATA CAN BE FOUND IN THE DIRECTORY idlprograms*/

void Central();

/* CALCULATE THE ELECTRON DISTRIBUTION AT  GIVEN Gamma[j] USING THE
   EXISTING INFORMATION FOR THE ELECTRON DISTRIBUTION AT THE PREVIOUS 
   TIME STEP, AND THE ALREADY CALCULATED ELECTRON DISTRIBUTION AT
   j+1, j+2, ... N1.*/

void CalcDistr(double *Ntemp,int j);

/*                             Synchrotron.c

CALCULATION OF THE SYNCHROTRON EMITTED POWER, FOLLOWING MELROSE'S GREEN'S 
FUNCTION the synchrotron power. THIS IS A SINGLE INTEGRATION OVER THE 
ELECTRON DISTRIBUTION*/

double Synchrotron(int v);

/* CALCULATION OF THE EXTERNAL COMPTON EMISSION. THIS IS A DOUBLE INTEGRATION
   OVER THE ELECTRON AND EXTERNAL PHOTON ENERGY DISTRIBUTIONS */

double ExtCompton(int v);

/*CALCULATION OF THE SSC EMISSION. THIS IS A DOUBLE INTEGRATION
OVER THE ELECTRON AND SOURCE PHOTON ENERGY DISTRIBUTIONS*/

double SSCemission(int v);

/*CALCULATION OF THE FIRST ORDER SSC EMISSION. THIS IS A DOUBLE INTEGRATION
OVER THE ELECTRON AND SOURCE PHOTON ENERGY DISTRIBUTIONS*/

double FirstCompton(int v);

/* INITIALIZES THE es ARRAY THAT HOLDS THE PHOTON ENERGIES
(IN UNITS OF ELECTRON REST MASS) IN WHICH THE CALCULATION OF THE EMISSION IS
PERFORMED. IT ALSO CALCULATES THE INDEX OF THE MAXIMUM ENERGY AT WHICH 
SYNCHROTRON ENERGY IS EMITTED.*/

void CalcEs();

/* CALCULATES A SERIES OF N2 INTEGRALS OF THE SOURCE PHOTON ENERGY DENSITY
IT ALSO CALCULATES THE PRODUCT  SRCfotonia[i]=Photons[i]*es[i], WHICH IS
USED IN CALCULATING THE LOSSES IN THE KLEIN-NISHINA REGIME.  */

void Photon_int();

// INTEGRATION OF TABULATED DATA 

double tab_int(double *y,  double dx, int i, int j);

//  INTEGRATION OF A FUNCTION USING GAUSSIAN QUADRATURE WITH N=64.

double gaussint64(double (*func)(double), double xmin, double xmax);

// THE FUNCTION c USED IN THE CALCULATION OF LOSSES

double c(double gammaprime);


/*-----------------------------------------------------------
   ARRAYS COMMON TO THE ENTIRE PROGRAM. THE ARRAYS WILL BE ALLOCATED 
   SPACE IN AllocateSpace.c.
   NOTE THAT THIS PROGRAM MAKES HEAVY USE OF GLOBAL ARRAYS AND VARIABLES,
   SO IF YOU WANT TO MODIFY IT, BE CAREFUL WHAT ARRAY OR VARIABLE YOU MODIFY.
   */

/* Gamma, THE ARRAY THAT HOLDS THE VALUES OF THE ELECTRON DISTRIBUTION 
   LORENTZ FACTORS, 
   GammaDot,THE ARRAY THAT HOLDS THE TIME DERIVATIVE OF Gamma
   DeltaGamma, THE ARRAY THAT HOLDS THE SPACINGS BETWEEN Gamma, 
   N, THE ARRAY THAT HOLDS THE ELECTRON ENERGY DISTRIBUTION AT A GIVEN TIME,
   Q, THE ARRAY THAT HOLDS THE ELECTRON INJECTION 
   Gamma_times_N,  THE ARRAY THAT HOLDS THE PRODUCT OF Gamma*N 
   Gamma_square, THE SQUARE OF Gamma*/

double *Gamma, *GammaDot, *DeltaGamma,*N,*Q,*Gamma_times_N,*Gamma_square;

/* es, THE ARRAY THAT HOLDS THE  PHOTON ENERGIES,
   Photons,  THE ARRAY THAT HOLDS THE PHOTON ENERGY DENSITY
   Synch, THE ARRAY THAT HOLDS THE SYNCHROTRON POWER, 
   EC,  THE ARRAY THAT HOLDS THE EC POWER
   SSC, THE ARRAY THAT HOLDS THE SSC POWER
   Efisrt, THE ARRAY THAT HOLDS THE FIRST ORDER SSC POWER
   SRCphotonint, THE ARRAY THAT HOLDS THE INTEGRATED  SOURCE 
   PHOTON ENERGY DENSITY UP TO  THE CORRESPONDING ENERGY. 
   SRCfotonia, THE ARRAY THAT HOLDS THE PRODUCT OF THE SOURCE PHOTON 
   ENERGY DENSITY TIMES THE PHOTON ENERGY. */

double *es,*Photons,*Synch,*EC, *SRCphotonint,*loges,*SSC,*Efirst,*SRCfotonia;

/* EXTe, THE ARRAY THAT HOLDS THE EXTERNAL PHOTON ENERGIES
   logEXTe=log(EXTe)
   Uext, THE ARRAY THAT HOLDS THE ENERGY DENSITY OF THE EXTERNAL PHOTON FIELD*/

double *EXTe, *logEXTe, *Uext;
 
/* ThomsonupperSRC, THE INDEX OF THE SOURCE PHOTON FIELD ENERGY UP TO 
   WHICH THE SCATTERING IS IN THE THOMSON REGIME FOR A GIVEN Gamma[N1]
  
   KNlowerSRC[N1], THE INDEX OF THE SOURCE PHOTON FIELD ENERGY ABOVE
   WHICH THE SCATTERING IS IN THE DEEP KN  REGIME FOR A GIVEN Gamma[N1]

   KNupperSRC[N1],  THE INDEX OF THE SOURCE PHOTON FIELD ENERGY UP TO
   WHICH THE PHOTON ENERGY IS LESS THAN Gamma[N1]
  
   ThomsonupperEXT[N1+1], THE INDEX OF THE EXTERNAL PHOTON FIELD ENERGY UP TO 
   WHICH THE SCATTERING IS IN THE THOMSON REGIME FOR A GIVEN Gamma[N1+1]

   KNlowerEXT[N1], THE INDEX OF THE EXTERNAL PHOTON FIELD ENERGY AT OR ABOVE
   WHICH THE SCATTERING IS IN THE DEEP KN  REGIME FOR A GIVEN Gamma[N1] 

   gamma_lower_index[N2],  THE INDEX OF THE SMALLEST Gamma
   THAT IS LARGER THAN A GIVEN es[i]

    EXTseed_upper  (F)  THE INDEX OF THE LARGEST ENTRY IN THE EXTERNAL PHOTON 
    FIELD THAT IS SMALLER THAN es[v] */

int *ThomsonupperSRC,*KNlowerSRC,*KNupperSRC,  *ThomsonupperEXT,
  *KNlowerEXT, *gamma_lower_index,*EXTseed_upper;

/* EXTphotonint, THE ARRAY THAT HOLDS THE INTEGRALS OF THE EXTERNAL 
   PHOTON ENERGY   DENSITY.
  
   EXTfotonia, THE ARRAY THAT HOLDS THE PRODUCT OF THE EXTERNAL PHOTON 
   ENERGY DENSITY TIMES THE PHOTON ENERGY. 
  
   THEY ARE BOTH  CALCULATED ONLY ONCE IN Initialize.c */

double  *EXTphotonint,*EXTfotonia;

/* CALCULATION OF  AN INTEGRAL OVER THE FINAL ELECTRON ENERGY FOR A GIVEN 
   INITIAL ELECTRON AND PHOTON ENERGY. THIS IS USED
   IN CALCULATING THE CONTINUOUS ENERGY LOSSES THAT ARE NOT IN THE 
   THOMSON REGIME. IT IS DONE TWICE ONCE FOR THE SOURCE PHOTON FIELD AND 
   ONCE FOR THE EXTERNAL PHOTON FIELD, AND THE  RESULTS ARE STORED IN 
   ContlossesSRC and ContlossesEXT.  */

double **ContlossesSRC, **ContlossesEXT;

/* EXTseed_lower (SRCseed_lower) IS A TWO DIMENSIONAL INTEGER ARRAY THAT 
   STORES THE INDEX OF THE MINIMUM EXT (SRC) SEED PHOTON ENERGY, 
   AS A FUNCTION OF THE ELECTRON ENERGY Gamma[j] AND FINAL PHOTON ENERGY es[i].
   NOTE THAT NOT THE WHOLE ARRAY IS FULL, BECAUSE ONLY SITUATIONS WITH 
   gamma[j]>es[i] ARE PLAUSIBLE*/

int **EXTseed_lower, **SRCseed_lower;

/* Csrc (Cext) IS A 3D ARRAY THAT HOLDS THE VALUES OF AN EXPRESSION USED 
   HEAVILY FOR THE CALCULATION OF THE COMPTON EMISSIVITIES. IT IS A FUNCTION 
   OF THE ENERGIES OF THE ELECTRON, THE INCOMING AND OUTGOING PHOTONS.
   THERE ARE TWO OF THOSE ARRAYS, ONE FOR THE EXTERNAL FIELD AND ONE FOR THE
   SOURCE FIELD*/

double ***Csrc, ***Cext;


/*-----------------------------------------------------------
  VARIABLES COMMON TO THE ENTIRE PROGRAM */

/* Injection - 1 FOR A POWER LAW,  2 FOR A DELTA FUNCTION
   N1 - NUMBER OF ELECTRON BINS
   N2 - NUMBER OF PHOTON BINS
   N3 - NUMBER OF EXTERNAL PHOTON BINS
   SynchIndex -  LAST es ELEMENT FOR WHICH SYNCHROTRON EMISSION IS CALCULATED
   PhotonField -  0 FOR THE CMB, 1 OTHERWISE
   jmin - THE INDEX jmin OF GammaMin IN THE Gamma ARRAY*/

int Injection,N1,N2,N3,SynchIndex,PhotonField,jmin;

/* B - MAGNETIC FIELD IN GAUSS
   UB - MAGNETIC FIELD ENERGY DENSITY
   Gamma0 - MINIMUM ELECTRON LORENTZ FACTOR FOLLOWED 
   GammaMax - MAXIMUM LORENTZ FACTOR OF INJECTED ELECTRONS AND
              ONLY POINT OF INJECTION FIR DELTA-FUNCTION INJECTION
   GammaMin - MINIMUM LORENTZ FACTOR OF INJECTED ELECTRONS
   timestep - THE SIZE OF THE TIMESTEP EMPLOYED 
   Luminosity - THE INJECTED ELECTRON POWER
   s - THE ELECTRON DISTRIBUTION INDEX 
   Gbulk - THE BULK LORENTZ FACTOR 
   delta - THE DOPPLER FACTOR*/

double B,UB,Gamma0,GammaMax,GammaMin,timestep,Luminosity,s,Gbulk,delta;

/* Const1 - A  CONSTANT USED IN Gamma_Dot, 4*SigmaT/(3*Me*CI)
   Const2 - A CONSTANT USED IN THE CALCULATION OF THE COMPTON EMISSION, 
   2*PI*pow(Re,2)*CI
   Const3 - A CONSTANT USED IN THE CALCULATION OF THE SYNCHROTRON EMISSIVITY,
   1.85*pow(2,.5)*pow(Qe,3)*B/Planck
   Z - THE REDSHIFT
   R - SOURCE RADIUS
   tesc - ELECTRON ESCAPE TIME
   esnorm -   THE LOGARITHMIC SPACING BETWEEN PHOTON BINS IN THE s  ARRAY
   gammanorm - THE LOGARITHMIC SPACING BETWEEN ELECTRON BINS IN THE Gamma ARRAY
   EXTnorm -  THE LOGARITHMIC SPACING BETWEEN PHOTON BINS IN THE EXTe  ARRAY
   Thomsonlimit - THE e0*gamma BELOW WHICH LOSSES ARE IN THE THOMSON REGIME
   KNlimit - THE e0*gamma BELOW WHICH LOSSES ARE IN THE DEEP KN REGIME
   U0 - THE ENERGY DENSITY OF THE EXTERNAL PHOTON FIELD
   epsilon0 - THE DIMENSIONLESS PEAK ENERGY OF THE EXTERNAL PHOTON DISTRIBUTION
   gaux,esaux - VARIABLES USED TO CARRY Gamma[j] AND es[i] FROM Gamma_dot.c 
   TO c.c 
   n_ratio_max - THE MAXIMUM VALUE OF THE ELECTRON DENSTITY RATIO IN THE 
   CURRENT AND PREVIOUS STEP. IT IS USED TO CALCULATE THE NEXT TIMESTEP
   t_steady - WHEN THE ADAPTIVE TIMESTEP BECOMES GREATER THAN t_steady IN UNITS
   OF tesc, STEADY STATE HAS BEEN REACHED*/

double Const1,Const2,Const3,Z,R,tesc,esnorm,gammanorm,EXTnorm,Thomsonlimit,KNlimit,U0,epsilon0, gaux,esaux,n_ratio_max,t_steady;
